#include <config.h>
#include <Ecore_Evas.h>
#include <Ecore_Config.h>
#include <Ecore_File.h>
#include <Esmart/Esmart_Container.h>
#include <Ewl.h>
#include "ewl_context_menu.h"
#include "points.h"
#include "Elitaire.h"
#include "eli_hiscore.h"
#include "elitaire_job.h"

struct Eli_App {
    Ecore_Evas        * ee;
    Evas_Object       * elitaire;
    Evas              * evas;
    Evas_Object       * gui;
    
    struct {
        struct{
            char      * current;
            Ecore_List* list;
        } gui,cards;
    } theme;
    
    struct {
        Evas_Bool       ctrl_down;
        Evas_Bool       alt_down;
    } key;
    
    struct {
        char          * game;
    } current, next;

    Ewl_Widget        * conmenu;
    //struct {
      //  Ewl_Widget    * w;            // The ewl_embed holding the conmenu
        //Evas_Object   * o;
//} embed;
};

/* The Config Types */
enum Config_Types {
    THEME,
    CARDS,
    VELOCITY,
    SHADOWS,
    ANIMATIONS,
    FRAME_RATE,
    LAZY
};

/* eli_app.cpp */
Eli_App *   eli_app_new(void);
void        eli_app_init(Eli_App* eap);
void        eli_app_del(Eli_App* eap);
void        eli_app_end(Eli_App * eap);

/* gui.cpp */
void         eli_app_gui_make(Eli_App* eap);
void         eli_app_gui_del(Eli_App* eap);
void         eli_app_gui_resize(Eli_App* eap,int w,int h);
/* menu.cpp */
void         eli_app_menu_container_fill(Eli_App * eap, Evas_Object * container);
void         eli_app_game_new(Eli_App * eap, const char * game);
void         eli_app_game_end(Eli_App * eap);
void         eli_app_velocity_set(Eli_App * eap, int velocity);
int          eli_app_velocity_get(Eli_App * eap);
void        _eli_app_new_game_cb(void *data, Evas_Object *o, const char *emission, const char *source);
/* theme.cpp */
Evas_Bool    edje_file_group_exists(const char * file, const char * group);
void         eli_app_theme_init(Eli_App * eap);
Ecore_List * eli_app_theme_names_get();
Ecore_List * eli_app_theme_dir_get();
void         eli_app_theme_change(Eli_App* eap, char * theme);
void         eli_app_theme_container_fill(Eli_App * eap, Evas_Object * container);
void         eli_app_cards_container_fill(Eli_App * eap, Evas_Object * container);
Evas_Bool    eli_app_theme_gui_exists(Eli_App * eap, const char * theme);
Evas_Bool    eli_app_theme_cards_exists(Eli_App * eap, const char * theme);
void         eli_app_theme_offset_get(Eli_App * eap, Evas_Coord * l, Evas_Coord * r, Evas_Coord * t, Evas_Coord * b);

/*win_config.cpp */
void         win_config_window_open(void);
/*win_about.cpp */
void         win_about_open(void);
/*win_hiscore.c */
void         win_highscore_open(const char * game);
void         win_highscore_add(Eli_App * eap, float points, pointsType type);
/* conmenu.cpp */
Ewl_Widget * eli_conmenu_new(Eli_App * eap);
/* eli_config.cpp */
void         eli_app_config_init(Eli_App * eap);

#define	eli_app_velocity_percent_to_real(percent) (200 + (int) (1800 * (percent)))
#define eli_app_velocity_real_to_percent(vel) (( (double) ((vel) - 200) )/1800.0)
#define eli_app_frame_rate_real_to_percent(fr) (( (double) ((fr) - 10) )/ 190.0)
#define	eli_app_frame_rate_percent_to_real(percent) (10 + (int) (190.0 * (percent)))

