#include <Ewl.h>
#include "ewl_context_menu.h"
#include <stdlib.h>

#define ABS(x) ( ((x) > 0) ? (x) : (-(x)) )

Ewl_Widget * ewl_context_menu_new(void)
{
    Ewl_Context_Menu * menu;
        
    menu = (Ewl_Context_Menu *) calloc(1,sizeof(Ewl_Context_Menu));
 
    if (!ewl_context_menu_init(menu)) {
        ewl_widget_destroy(EWL_WIDGET(menu));
        menu = NULL;
    }
        
    return EWL_WIDGET(menu);
}

int ewl_context_menu_init(Ewl_Context_Menu * menu)
{
    /* init the floater */
    if (!ewl_floater_init(EWL_FLOATER(menu))) return 0;
    /* set up the menu */
    menu->menu = ewl_menu_new();

    ewl_button_label_set(EWL_BUTTON(menu->menu), " ");
    ewl_container_child_append(EWL_CONTAINER(menu), menu->menu);
    ewl_widget_show(menu->menu);
    
    ewl_container_redirect_set(EWL_CONTAINER(menu), EWL_CONTAINER(menu->menu));
    
    ewl_widget_inherit(EWL_WIDGET(menu), "context_menu");
    return 1;
}

void ewl_context_menu_active_area_add(Ewl_Context_Menu * m, Ewl_Widget * p)
{    
    ewl_callback_append(p, EWL_CALLBACK_MOUSE_DOWN, ewl_context_menu_mouse_down_cb, m);
}

void ewl_context_menu_active_area_del(Ewl_Context_Menu * m, Ewl_Widget * p)
{
    ewl_callback_del(p, EWL_CALLBACK_MOUSE_DOWN, ewl_context_menu_mouse_down_cb);
}

void ewl_context_menu_mouse_down_cb(Ewl_Widget * w, void * ev_data, void * user_data)
{
    Ewl_Context_Menu * cm;
    Ewl_Event_Mouse_Down * ev;
    
    ev = ev_data;
    
    if (ev->button == 3) 
    {
        int w, h;
        int px, py;
        Ewl_Widget * follow;

        cm = EWL_CONTEXT_MENU(user_data);
        follow = ewl_floater_follow_get(EWL_FLOATER(cm));

        if (follow) {
            px = ewl_object_current_x_get(EWL_OBJECT(follow));
            py = ewl_object_current_y_get(EWL_OBJECT(follow));  
        }
        else {
            px = 0;
            py = 0;
        }
        
        w = ewl_object_current_w_get(EWL_OBJECT(cm->menu));
        h = ewl_object_current_h_get(EWL_OBJECT(cm->menu));    
        ewl_widget_show(EWL_WIDGET(cm));
        ewl_floater_position_set(EWL_FLOATER(cm), 
                                                    ev->x - px, 
                                                    ev->y - py - h);
        ewl_callback_call(EWL_WIDGET(cm->menu), EWL_CALLBACK_FOCUS_IN);
        ewl_object_state_remove(EWL_OBJECT(cm), EWL_FLAG_STATE_PRESSED);

    } 
}


