#include"Esmart_Resize.h"
#include <Esmart/Esmart_Draggies.h>
#include"Eli_App.h"
#include<iostream>

/* internal functions */
static void _eli_app_gui_swallow             (Evas_Object * gui, const char * part, Evas_Object * o);

/* callback  declaration */
static void _quit_cb                         (void * data, Evas_Object * o, const char * emission, const char * source);
static void _velocity_drag_stop_cb           (void * data, Evas_Object * o, const char * emission, const char * source);
static void _velocity_drag_cb                (void * data, Evas_Object * o, const char * emission, const char * source);
static void _undo_cb                         (void * data, Evas_Object * o, const char * emission, const char * source);
static void _restart_cb                      (void * data, Evas_Object * o, const char * emission, const char * source);
static void _elitaire_scroll_stop_cb         (void * data, Evas_Object * o, const char * emission, const char * source);
static void _elitaire_scroll_left_start_cb   (void * data, Evas_Object * o, const char * emission, const char * source);
static void _elitaire_scroll_right_start_cb  (void * data, Evas_Object * o, const char * emission, const char * source);
static void _mouse_up_cb                     (void * data, Evas * e, Evas_Object * obj, void * event_info);
static void _mouse_down_cb                   (void * data, Evas * e, Evas_Object * obj, void * event_info);
static void _key_down_cb                     (void * data, Evas * e, Evas_Object * obj, void * event_info);
static void _key_up_cb                       (void * data, Evas * e, Evas_Object * obj, void * event_info);

/* externals */
void eli_app_gui_make(Eli_App * eap)
{
    Evas * evas;
    Evas_Coord w, h;
    char * gui_theme;
    const char * shaped;
    const char * borderless;
    int is_shaped;

    is_shaped = 0;
    evas = eap->evas;
    eap->gui = edje_object_add(evas);
    gui_theme =
        ecore_config_theme_with_path_from_name_get(eap->theme.gui.current);

    if (edje_object_file_set(eap->gui, gui_theme, "main")) {
        evas_object_move(eap->gui, 0, 0);
        evas_object_repeat_events_set(eap->gui, 1);
        edje_object_size_min_get(eap->gui, &w, &h);
        evas_object_resize(eap->gui, w, h);
        //evas_object_layer_set(eap->gui, 1000);
        evas_object_focus_set(eap->gui, 1);
        edje_object_signal_callback_add(eap->gui, "quit", "*", _quit_cb, eap);
        edje_object_signal_callback_add(eap->gui, "undo", "*", _undo_cb, eap);
        edje_object_signal_callback_add(eap->gui, "restart", "*", _restart_cb,
                                        eap);
        edje_object_signal_callback_add(eap->gui, "new_game", "*",
                                        _eli_app_new_game_cb, eap);
        evas_object_event_callback_add(eap->gui, EVAS_CALLBACK_KEY_DOWN,
                                       _key_down_cb, eap);
        evas_object_event_callback_add(eap->gui, EVAS_CALLBACK_KEY_UP,
                                       _key_up_cb, eap);
        evas_object_show(eap->gui);
    }
    else {
        fprintf(stderr, _("Elitaire Error: unable to find %s theme!\n"),
                gui_theme);
        return;
    }

    /* some ecore_evas stuff */
    shaped = edje_object_data_get(eap->gui, "shaped");
    borderless = edje_object_data_get(eap->gui, "borderless");

    if (!shaped) ecore_evas_shaped_set(eap->ee, 0);
    else if (!strcmp("1", shaped)) {
        ecore_evas_shaped_set(eap->ee, 1);
        is_shaped = 1;
    }
    else ecore_evas_shaped_set(eap->ee, 0);

    if (!borderless) ecore_evas_borderless_set(eap->ee, 0);
    else if (!strcmp("1", borderless)) ecore_evas_borderless_set(eap->ee, 1);
    else ecore_evas_borderless_set(eap->ee, 0);

    ecore_evas_resize(eap->ee, w, h);
    ecore_evas_size_min_set(eap->ee, w, h);

    /* the card bar */
    if (edje_object_part_exists(eap->gui, "elitaire_cards_box")) {
        Evas_Object * container;

        container = esmart_container_new(evas);
        esmart_container_direction_set(container,
                                       CONTAINER_DIRECTION_VERTICAL);
        esmart_container_alignment_set(container, CONTAINER_ALIGN_TOP);
        evas_object_show(container);

        _eli_app_gui_swallow(eap->gui, "elitaire_cards_box", container);
        eli_app_cards_container_fill(eap, container);

        edje_object_signal_callback_add(eap->gui, "scroll,stop",
                                        "elitaire_cards_box",
                                        _elitaire_scroll_stop_cb, container);
        edje_object_signal_callback_add(eap->gui, "scroll,left,start",
                                        "elitaire_cards_box",
                                        _elitaire_scroll_left_start_cb,
                                        container);
        edje_object_signal_callback_add(eap->gui, "scroll,right,start",
                                        "elitaire_cards_box",
                                        _elitaire_scroll_right_start_cb,
                                        container);
    }

    /* the theme bar */
    if (edje_object_part_exists(eap->gui, "elitaire_theme_box")) {
        Evas_Object * container;

        container = esmart_container_new(evas);
        esmart_container_direction_set(container,
                                       CONTAINER_DIRECTION_VERTICAL);
        esmart_container_alignment_set(container, CONTAINER_ALIGN_TOP);
        evas_object_show(container);

        _eli_app_gui_swallow(eap->gui, "elitaire_theme_box", container);
        eli_app_theme_container_fill(eap, container);

        edje_object_signal_callback_add(eap->gui, "scroll,stop",
                                        "elitaire_theme_box",
                                        _elitaire_scroll_stop_cb, container);
        edje_object_signal_callback_add(eap->gui, "scroll,left,start",
                                        "elitaire_theme_box",
                                        _elitaire_scroll_left_start_cb,
                                        container);
        edje_object_signal_callback_add(eap->gui, "scroll,right,start",
                                        "elitaire_theme_box",
                                        _elitaire_scroll_right_start_cb,
                                        container);
    }

    /*the menu bar */
    if (edje_object_part_exists(eap->gui, "elitaire_new_box")) {
        Evas_Object * container;

        container = esmart_container_new(evas);
        esmart_container_direction_set(container,
                                       CONTAINER_DIRECTION_VERTICAL);
        esmart_container_alignment_set(container, CONTAINER_ALIGN_TOP);
        evas_object_show(container);

        _eli_app_gui_swallow(eap->gui, "elitaire_new_box", container);
        eli_app_menu_container_fill(eap, container);

        edje_object_signal_callback_add(eap->gui, "scroll,stop",
                                        "elitaire_new_box",
                                        _elitaire_scroll_stop_cb, container);
        edje_object_signal_callback_add(eap->gui, "scroll,left,start",
                                        "elitaire_new_box",
                                        _elitaire_scroll_left_start_cb,
                                        container);
        edje_object_signal_callback_add(eap->gui, "scroll,right,start",
                                        "elitaire_new_box",
                                        _elitaire_scroll_right_start_cb,
                                        container);

    }
    /* for the velocity control; perhaps it will be removed */
    edje_object_signal_callback_add(eap->gui, "drag,stop",
                                    "elitaire_velocity_slider",
                                    _velocity_drag_stop_cb, eap);
    edje_object_signal_callback_add(eap->gui, "drag",
                                    "elitaire_velocity_slider",
                                    _velocity_drag_cb, eap);
    _velocity_drag_cb(eap, eap->gui, "drag", "elitaire_velocity_slider");

    /* theme parts for moving the window */
    if (edje_object_part_exists(eap->gui, "elitaire_drag")) {
        Evas_Object * o;
        
        o = esmart_draggies_new(eap->ee);
        esmart_draggies_button_set(o, 1);
        _eli_app_gui_swallow(eap->gui, "elitaire_drag", o);
    }

    const char *resize_types[] = {
        "elitaire_resize_left",
        "elitaire_resize_right",
        "elitaire_resize_top",
        "elitaire_resize_bottom",
        "elitaire_resize_left_top",
        "elitaire_resize_right_top",
        "elitaire_resize_left_bottom",
        "elitaire_resize_right_bottom"
    };

    Esmart_Resize_Type rt[] = {
        ESMART_RESIZE_LEFT,
        ESMART_RESIZE_RIGHT,
        ESMART_RESIZE_TOP,
        ESMART_RESIZE_BOTTOM,
        ESMART_RESIZE_LEFT_TOP,
        ESMART_RESIZE_RIGHT_TOP,
        ESMART_RESIZE_LEFT_BOTTOM,
        ESMART_RESIZE_RIGHT_BOTTOM
    };

    /* theme parts for resizing the window */
    for (int i = 0; i < 8; i++) {
        if (edje_object_part_exists(eap->gui, resize_types[i])) {
            Evas_Object * o;
            
            o = esmart_resize_new(eap->ee);
            esmart_resize_button_set(o, 1);
            esmart_resize_type_set(o, rt[i]);
            _eli_app_gui_swallow(eap->gui, resize_types[i], o);
        }
    }

    /* theme parts for the context menu */
    if (edje_object_part_exists(eap->gui, "elitaire_context_menu")) {
        Evas_Object * o;
        Ewl_Widget * w;

        w = ewl_embed_new();
        ewl_object_fill_policy_set(EWL_OBJECT(w), EWL_FLAG_FILL_FILL);
        o = ewl_embed_evas_set(EWL_EMBED(w), evas, 
			(void **) ecore_evas_software_x11_window_get(eap->ee));
        _eli_app_gui_swallow(eap->gui, "elitaire_context_menu", o);
    
	eap->conmenu = eli_conmenu_new(eap);
	ewl_container_child_append(EWL_CONTAINER(w), eap->conmenu);
	ewl_context_menu_active_area_add(EWL_CONTEXT_MENU(eap->conmenu), w);
    }

    {
        int vel;
        
        vel = ecore_config_int_get("velocity");
        eli_app_velocity_set(eap, vel);
    }

    if (eap->elitaire)
        _eli_app_gui_swallow(eap->gui, "elitaire_card_field", eap->elitaire);
}

void eli_app_gui_del(Eli_App * eap)
{
    const char * unsw[] = {
        "elitaire_context_menu",
        "elitaire_drag",
        "elitaire_resize_left",
        "elitaire_resize_right",
        "elitaire_resize_top",
        "elitaire_resize_bottom",
        "elitaire_resize_left_top",
        "elitaire_resize_right_top",
        "elitaire_resize_left_bottom",
        "elitaire_resize_right_bottom",
        "elitaire_cards_box",
        "elitaire_theme_box",
        "elitaire_new_box"
    };

    for (int i = 0; i < 13; i++) {
        Evas_Object * o;
        
        o = edje_object_part_swallow_get(eap->gui, unsw[i]);
        if (o) {
            edje_object_part_unswallow(eap->gui, o);
            evas_object_del(o);
        }
    }

    if (edje_object_part_swallow_get(eap->gui, "elitaire_card_field"))
        edje_object_part_unswallow(eap->gui, eap->elitaire);

    evas_object_del(eap->gui);
    eap->gui = NULL;
}

void eli_app_gui_resize(Eli_App * eap, int w, int h)
{
    evas_object_resize(eap->gui, (Evas_Coord) w, (Evas_Coord) h);
}

/* internal */
static void _eli_app_gui_swallow(Evas_Object * gui, const char *part,
                          Evas_Object * o)
{
    Evas_Coord x, y, w, h;

    edje_object_part_geometry_get(gui, part, &x, &y, &w, &h);
    evas_object_move(o, x, y);
    evas_object_resize(o, w, h);
    evas_object_show(o);
    edje_object_part_swallow(gui, part, o);
}

/* callbacks */
static void _quit_cb(void * data, Evas_Object * o, const char * emission,
              const char * source)
{
    ecore_main_loop_quit();
}

static void _velocity_drag_stop_cb(void * data, Evas_Object * o, const char * emission,
                            const char * source)
{
    Eli_App * eap;

    eap = (Eli_App *) data;

    ecore_config_int_set("velocity", eli_app_velocity_get(eap));
}

static void _velocity_drag_cb(void * data, Evas_Object * o, const char * emission,
                       const char * source)
{
    Eli_App * eap;

    eap = (Eli_App *) data;

    if (edje_object_part_exists(o, "elitaire_velocity_value") && eap) {
        char buffer[5];

        snprintf(buffer, 5, "%i", eli_app_velocity_get(eap));
        edje_object_part_text_set(o, "elitaire_velocity_value", buffer);
    }
}

static void _undo_cb(void * data, Evas_Object * o, const char * emission,
              const char * source)
{
    Eli_App * eap;

    eap = (Eli_App *) data;

    if (!eap) return;
    if (eap->elitaire) elitaire_object_undo(eap->elitaire);
}

static void _restart_cb(void * data, Evas_Object * o, const char * emission,
                 const char * source)
{
    Eli_App * eap;

    eap = (Eli_App *) data;

    if (!eap) return;
    if (eap->elitaire) elitaire_object_restart(eap->elitaire);
}

/* FIXME: what i am good for? */
static void _mouse_up_cb(void * data, Evas * e, Evas_Object * obj, void * event_info)
{
    Eli_App *eap;

    if (data) eap = (Eli_App *) data;
}

/*
static void _mouse_down_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
{
    Eli_App * eap;
    Evas_Event_Mouse_Down * ev;
    int x, y;

    if (!data) return;

    eap = (Eli_App *) data;
    ev = (Evas_Event_Mouse_Down *) event_info;

    switch (ev->button) {
    case 3:
        ecore_evas_geometry_get(eap->ee, &x, &y, NULL, NULL);
        x += evas_coord_world_x_to_screen(e, ev->canvas.x);
        y += evas_coord_world_y_to_screen(e, ev->canvas.y);
        ewl_context_menu_move(EWL_CONTEXT_MENU(eap->conmenu), x, y);
        ewl_widget_show(eap->conmenu);
        break;
    
    default:
        ewl_widget_hide(eap->conmenu);
        break;
    }
}
*/
static void _elitaire_scroll_stop_cb(void * data, Evas_Object * o,
                              const char * emission, const char * source)
{
    Evas_Object * container;

    container = (Evas_Object *) data;
    esmart_container_scroll_stop(container);
}

static void _elitaire_scroll_left_start_cb(void * data, Evas_Object * o,
                                    const char * emission, const char * source)
{
    Evas_Object * container;

    container = (Evas_Object *) data;
    esmart_container_scroll_start(container, -2.0);
}

static void _elitaire_scroll_right_start_cb(void * data, Evas_Object * o,
                                     const char * emission, const char * source)
{
    Evas_Object * container;

    container = (Evas_Object *) data;
    esmart_container_scroll_start(container, 2.0);
}

static void _key_down_cb(void * data, Evas * e, Evas_Object * obj, void * event_info)
{
    Eli_App *eap;
    Evas_Event_Key_Down *ev;

    eap = (Eli_App *) data;
    ev = (Evas_Event_Key_Down *) event_info;
    
    printf("You hit key: %s \n", ev->key);

    if (!strcmp(ev->key, "Control_L") || !strcmp(ev->key, "Control_R")) {
        eap->key.ctrl_down = true;
        return;
    }
    if (!strcmp(ev->key, "Alt_L")) {
        eap->key.alt_down = true;
        return;
    }

    if (eap->key.ctrl_down && !eap->key.alt_down) {
        if (!strcmp(ev->key, "z")) edje_object_signal_emit(eap->gui, "undo", "");
        else if (!strcmp(ev->key, "r")) edje_object_signal_emit(eap->gui, "restart", "");
        else if (!strcmp(ev->key, "n")) edje_object_signal_emit(eap->gui, "new_game", "");
        else if (!strcmp(ev->key, "a")) {
            if (ecore_evas_alpha_get(eap->ee)) {
                ecore_evas_alpha_set(eap->ee, 0);
                edje_object_signal_emit(eap->gui, "elitaire,alpha,off", "");
            }
            else {
                ecore_evas_alpha_set(eap->ee, 1);
                edje_object_signal_emit(eap->gui, "elitaire,alpha,on", "");
            }
        }
    }
    
}

static void _key_up_cb(void *data, Evas * e, Evas_Object * obj, void *event_info)
{
    Eli_App *eap;
    Evas_Event_Key_Up * ev;

    eap = (Eli_App *) data;
    ev = (Evas_Event_Key_Up *) event_info;

    if (!strcmp(ev->key, "Control_L")) eap->key.ctrl_down = false;
    else if (!strcmp(ev->key, "Control_R")) eap->key.ctrl_down = false;
    else if (!strcmp(ev->key, "Alt_L")) eap->key.alt_down = false;
}
