#include"Eli_App.h"
#include <eli_statistics.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>

static Ewl_Widget * wh_entry = NULL;
static Eli_App    * wh_eap = NULL;
static float        wh_points;
static pointsType   wh_type;

static Ewl_Widget * _win_highscore_page_new   (char * game);
static void         _destroy_cb               (Ewl_Widget * w, void * event, void * data);
static void         _name_destroy_cb          (Ewl_Widget * w, void * event, void * data);
static void         _win_highscore_clicked_cb (Ewl_Widget * w, void * event, void * data);

/*
 * External API
 */

void win_highscore_open(const char * game)
{
    Ewl_Widget * win;
    Ewl_Widget * o;
    Ewl_Widget * notebook;
    const char ** games;
    int i;
    Ewl_Widget * selected_page = NULL;

    /* Setup and show the hiscore window */
    win = ewl_dialog_new();
    ewl_dialog_action_position_set(EWL_DIALOG(win), EWL_POSITION_BOTTOM);
    ewl_window_title_set(EWL_WINDOW(win), _("Highscore"));
    ewl_window_name_set(EWL_WINDOW(win), "Elitaire");
    ewl_window_class_set(EWL_WINDOW(win), "Elitaire");
    //ewl_object_size_request(EWL_OBJECT(win), 200, 200);
    ewl_callback_append(win, EWL_CALLBACK_DELETE_WINDOW, _destroy_cb,
                        win);
    ewl_dialog_has_separator_set(EWL_DIALOG(win), 1);
    ewl_object_fill_policy_set(EWL_OBJECT(win), EWL_FLAG_FILL_FILL);
    ewl_widget_show(win);

    /* Setup and show the stock icons */
    ewl_dialog_active_area_set(EWL_DIALOG(win), EWL_POSITION_BOTTOM);
    o = ewl_button_new();
    ewl_button_stock_type_set(EWL_BUTTON(o), EWL_STOCK_OK);
    ewl_container_child_append(EWL_CONTAINER(win), o);
    ewl_callback_append(o, EWL_CALLBACK_CLICKED, _destroy_cb, win);
    ewl_widget_show(o);

    /* Setup and show th notebook */
    ewl_dialog_active_area_set(EWL_DIALOG(win), EWL_POSITION_TOP);
    notebook = ewl_notebook_new();
    ewl_notebook_tabbar_position_set(EWL_NOTEBOOK(notebook),
                                     EWL_POSITION_LEFT);
    ewl_container_child_append(EWL_CONTAINER(win), notebook);
    ewl_object_fill_policy_set(EWL_OBJECT(notebook), EWL_FLAG_FILL_HFILL);
    //  ewl_notebook_tabs_alignment_set(EWL_NOTEBOOK(notebook), EWL_FLAG_FILL_FILL);
    ewl_widget_show(notebook);

    games = elitaire_available_games_get();

    for (i = 0; games[i][0]; i++) {
        Ewl_Widget * page;
        char * labeltext;

        labeltext = strdup(games[i]);

        page = _win_highscore_page_new(labeltext);
        ewl_widget_show(page);
        ewl_container_child_append(EWL_CONTAINER(notebook), page);
        ewl_notebook_page_tab_text_set(EWL_NOTEBOOK(notebook), page,
                                       labeltext);

        if (!strcmp(labeltext, game)) selected_page = page;
        free(labeltext);
    }

    if (selected_page)
        ewl_notebook_visible_page_set(EWL_NOTEBOOK(notebook), selected_page);
}

void win_highscore_add(Eli_App * eap, float points, pointsType type)
{
    Ewl_Widget * win;
    Ewl_Widget * o;
    Ewl_Widget * vbox;
    char * username;

    /* when thers is already a name window open dont open a new one */
    if (wh_entry) return;
    /* a hopefully good default */
    username = getpwuid(geteuid())->pw_name;
    /* Setup and show the name window */
    win = ewl_dialog_new();
    ewl_dialog_action_position_set(EWL_DIALOG(win), EWL_POSITION_BOTTOM);
    ewl_window_title_set(EWL_WINDOW(win), _("Name"));
    ewl_window_name_set(EWL_WINDOW(win), "Elitaire");
    ewl_window_class_set(EWL_WINDOW(win), "Elitaire");
    ewl_object_size_request(EWL_OBJECT(win), 20, 20);
    ewl_callback_append(win, EWL_CALLBACK_DELETE_WINDOW, _name_destroy_cb,
                        win);
    ewl_dialog_has_separator_set(EWL_DIALOG(win), 1);
    ewl_object_fill_policy_set(EWL_OBJECT(win), EWL_FLAG_FILL_FILL);
    ewl_widget_show(win);

    /* Setup and show the stock icons */
    ewl_dialog_active_area_set(EWL_DIALOG(win), EWL_POSITION_BOTTOM);
    o = ewl_button_new();
    ewl_button_stock_type_set(EWL_BUTTON(o), EWL_STOCK_OK);
    ewl_container_child_append(EWL_CONTAINER(win), o);
    ewl_callback_append(o, EWL_CALLBACK_CLICKED, _win_highscore_clicked_cb,
                        win);
    ewl_widget_show(o);

    /* vbox for text and entry */
    ewl_dialog_active_area_set(EWL_DIALOG(win), EWL_POSITION_TOP);
    vbox = ewl_vbox_new();
    ewl_container_child_append(EWL_CONTAINER(win), vbox);
    ewl_widget_show(vbox);

    /* Text */
    o = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(o), _("Please insert your name"));
    ewl_container_child_append(EWL_CONTAINER(vbox), o);
    ewl_object_fill_policy_set(EWL_OBJECT(o), EWL_FLAG_FILL_FILL);
    //ewl_object_alignment_set(EWL_OBJECT(o), EWL_FLAG_ALIGN_CENTER);
    //ewl_text_align_set(EWL_TEXT(o), EWL_FLAG_ALIGN_CENTER);
    //ewl_object_padding_set(EWL_OBJECT(o),15,15,15,2);
    ewl_widget_show(o);

    /* Entry */
    o = ewl_entry_new();
    ewl_text_text_set(EWL_TEXT(o), username);
    ewl_container_child_append(EWL_CONTAINER(vbox), o);
    ewl_object_fill_policy_set(EWL_OBJECT(o), EWL_FLAG_FILL_FILL);
    //ewl_object_alignment_set(EWL_OBJECT(o), EWL_FLAG_ALIGN_CENTER);
    ewl_text_align_set(EWL_TEXT(o), EWL_FLAG_ALIGN_CENTER);
    ewl_object_padding_set(EWL_OBJECT(o), 15, 15, 2, 15);
    ewl_widget_show(o);

    wh_entry = o;
    wh_points = points;
    wh_eap = eap;
    wh_type = type;
}

/*
 * Internals
 */
static void _destroy_cb(Ewl_Widget * w, void * event, void * data)
{
    Ewl_Widget * win;

    win = (Ewl_Widget *) data;
    ewl_widget_destroy(win);
    
    wh_eap = NULL;
    wh_entry = NULL;
}

static void _name_destroy_cb(Ewl_Widget * w, void * event, void * data)
{
    Ewl_Widget * win;

    win = (Ewl_Widget *) data;
    ewl_widget_destroy(win);
    
    eli_app_game_end(wh_eap);
    
    wh_eap = NULL;
    wh_entry = NULL;
}

static Ewl_Widget * _win_highscore_page_new(char * game)
{
    Ewl_Widget * page;
    Ewl_Widget * sbox;
    Ewl_Widget * hbox;
    Ewl_Widget * vbox[3];
    Ewl_Widget * w;
    Evas_List * l, * list;
    pointsType ptype;
    Eli_Statistics stats;
    char buffer[5];
    float qoute;

    int i;

    /* the page */
    page = ewl_border_new();
    ewl_border_text_set(EWL_BORDER(page), game);
    ewl_object_padding_set(EWL_OBJECT(page), 15, 15, 5, 15);
    /* the statistics */
    sbox = ewl_hbox_new();
    ewl_container_child_append(EWL_CONTAINER(page), sbox);
    ewl_object_padding_set(EWL_OBJECT(sbox), 5, 5, 5, 0);
    ewl_object_fill_policy_set(EWL_OBJECT(sbox), EWL_FLAG_FILL_HFILL);
    ewl_widget_show(sbox);
    stats = eli_statistics_get(game);

    vbox[0] = ewl_vbox_new();
    ewl_container_child_append(EWL_CONTAINER(sbox), vbox[0]);
    ewl_widget_show(vbox[0]);
    vbox[1] = ewl_vbox_new();
    ewl_container_child_append(EWL_CONTAINER(sbox), vbox[1]);
    ewl_widget_show(vbox[1]);
    ewl_object_fill_policy_set(EWL_OBJECT(vbox[1]), EWL_FLAG_FILL_VFILL);
    w = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(w), _("Won Games:"));
    ewl_container_child_append(EWL_CONTAINER(vbox[0]), w);
    ewl_widget_show(w);
    w = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(w), _("Lost Games:"));
    ewl_container_child_append(EWL_CONTAINER(vbox[0]), w);
    ewl_widget_show(w);
    w = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(w), _("relative Quote:"));
    ewl_container_child_append(EWL_CONTAINER(vbox[0]), w);
    ewl_widget_show(w);
    snprintf(buffer, 5, "%i", stats.won_games);
    w = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(w), buffer);
    ewl_container_child_append(EWL_CONTAINER(vbox[1]), w);
    ewl_widget_show(w);
    snprintf(buffer, 5, "%i", stats.lost_games);
    w = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(w), buffer);
    ewl_container_child_append(EWL_CONTAINER(vbox[1]), w);
    ewl_widget_show(w);
    if (stats.won_games == 0 && stats.lost_games == 0)
        qoute = 0.0;
    else
        qoute =
            stats.won_games / (float) (stats.won_games + stats.lost_games);
    snprintf(buffer, 5, "%.2f", qoute);
    w = ewl_text_new();
    ewl_text_text_set(EWL_TEXT(w), buffer);
    ewl_container_child_append(EWL_CONTAINER(vbox[1]), w);
    ewl_widget_show(w);

    /* table */
    hbox = ewl_hbox_new();
    ewl_container_child_append(EWL_CONTAINER(page), hbox);
    ewl_object_padding_set(EWL_OBJECT(hbox), 5, 5, 5, 0);
    ewl_object_fill_policy_set(EWL_OBJECT(hbox), EWL_FLAG_FILL_HFILL);
    ewl_widget_show(hbox);

    for (i = 0; i < 3; i++) {
        vbox[i] = ewl_vbox_new();
        ewl_container_child_append(EWL_CONTAINER(hbox), vbox[i]);
        ewl_widget_show(vbox[i]);
    }
    ewl_object_fill_policy_set(EWL_OBJECT(vbox[0]), EWL_FLAG_FILL_VFILL);
    ewl_object_fill_policy_set(EWL_OBJECT(vbox[1]), EWL_FLAG_FILL_FILL);
    ewl_object_fill_policy_set(EWL_OBJECT(vbox[2]), EWL_FLAG_FILL_VFILL);

    /* insert the table entries */
    list = eli_highscore_get(game);
    ptype = eli_highscore_points_type_get(game);
    l = list;
    i = 1;

    while (l) {
        Eli_Highscore_Entry * en;
        Ewl_Widget * w0, * w1, * w2;
        char * pstring;
        char bufindex[5];

        en = (Eli_Highscore_Entry *) evas_list_data(l);
        pstring = pointsType_point_string_get(en->points, ptype);
        //snprintf(buffer, sizeof(buffer), "%i", (int) en->points);
        snprintf(bufindex, sizeof(bufindex), "%i.", i);

        w0 = ewl_text_new();
        ewl_text_text_set(EWL_TEXT(w0), bufindex);
        ewl_widget_show(w0);
        w1 = ewl_text_new();
        ewl_object_fill_policy_set(EWL_OBJECT(w1), EWL_FLAG_FILL_NONE);
        ewl_object_alignment_set(EWL_OBJECT(w1), EWL_FLAG_ALIGN_CENTER);
        ewl_text_align_set(EWL_TEXT(w1), EWL_FLAG_ALIGN_CENTER);
        ewl_text_text_append(EWL_TEXT(w1), en->username);
        ewl_widget_show(w1);
        w2 = ewl_text_new();
        ewl_text_align_set(EWL_TEXT(w2), EWL_FLAG_ALIGN_RIGHT);
        ewl_text_text_append(EWL_TEXT(w2), pstring);
        ewl_widget_show(w2);

        free(pstring);

        ewl_container_child_append(EWL_CONTAINER(vbox[0]), w0);
        ewl_container_child_append(EWL_CONTAINER(vbox[1]), w1);
        ewl_container_child_append(EWL_CONTAINER(vbox[2]), w2);

        l = l->next;
        i++;
    }
    eli_highscore_free(list);

    return page;
}

static void _win_highscore_clicked_cb(Ewl_Widget * w, void * event, void * data)
{
    Ewl_Widget * win;
    char * text;

    win = (Ewl_Widget *) data;

    text = ewl_text_text_get(EWL_TEXT(wh_entry));
    eli_highscore_entry_add(wh_eap->current.game, text, wh_points,
                            wh_type);
    win_highscore_open(wh_eap->current.game);

    ewl_widget_destroy(win);

    eli_app_game_end(wh_eap);
}
